@@ -6,12 +6,26 @@ from typing import (
     Any, Dict, Iterable, Iterator, List, Optional, Set, Union,
 )
 
+from amundsen_rds.models import RDSModel
+from amundsen_rds.models.cluster import Cluster as RDSCluster
+from amundsen_rds.models.column import (
+    ColumnBadge as RDSColumnBadge, ColumnDescription as RDSColumnDescription, TableColumn as RDSTableColumn,
+)
+from amundsen_rds.models.database import Database as RDSDatabase
+from amundsen_rds.models.schema import Schema as RDSSchema
+from amundsen_rds.models.table import (
+    Table as RDSTable, TableDescription as RDSTableDescription,
+    TableProgrammaticDescription as RDSTableProgrammaticDescription, TableTag as RDSTableTag,
+)
+from amundsen_rds.models.tag import Tag as RDSTag
+
 from databuilder.models.badge import Badge, BadgeMetadata
 from databuilder.models.cluster import cluster_constants
 from databuilder.models.graph_node import GraphNode
 from databuilder.models.graph_relationship import GraphRelationship
 from databuilder.models.graph_serializable import GraphSerializable
 from databuilder.models.schema import schema_constant
+from databuilder.models.table_serializable import TableSerializable
 
 DESCRIPTION_NODE_LABEL_VAL = 'Description'
 DESCRIPTION_NODE_LABEL = DESCRIPTION_NODE_LABEL_VAL
@@ -27,7 +41,7 @@ def _format_as_list(tags: Union[List, str, None]) -> List:
     return tags
 
 
-class TagMetadata(GraphSerializable):
+class TagMetadata(GraphSerializable, TableSerializable):
     TAG_NODE_LABEL = 'Tag'
     TAG_KEY_FORMAT = '{tag}'
     TAG_TYPE = 'tag_type'
@@ -42,8 +56,9 @@ class TagMetadata(GraphSerializable):
                  ):
         self._name = name
         self._tag_type = tag_type
-        self._nodes = iter([self.create_tag_node(self._name, self._tag_type)])
-        self._relations: Iterator[GraphRelationship] = iter([])
+        self._nodes = self._create_node_iterator()
+        self._relations = self._create_relation_iterator()
+        self._records = self._create_record_iterator()
 
     @staticmethod
     def get_tag_key(name: str) -> str:
@@ -51,17 +66,23 @@ class TagMetadata(GraphSerializable):
             return ''
         return TagMetadata.TAG_KEY_FORMAT.format(tag=name)
 
-    @staticmethod
-    def create_tag_node(name: str, tag_type: str = DEFAULT_TYPE) -> GraphNode:
+    def get_node(self) -> GraphNode:
         node = GraphNode(
-            key=TagMetadata.get_tag_key(name),
+            key=TagMetadata.get_tag_key(self._name),
             label=TagMetadata.TAG_NODE_LABEL,
             attributes={
-                TagMetadata.TAG_TYPE: tag_type
+                TagMetadata.TAG_TYPE: self._tag_type
             }
         )
         return node
 
+    def get_record(self) -> RDSModel:
+        record = RDSTag(
+            rk=TagMetadata.get_tag_key(self._name),
+            tag_type=self._tag_type
+        )
+        return record
+
     def create_next_node(self) -> Optional[GraphNode]:
         # return the string representation of the data
         try:
@@ -76,9 +97,27 @@ class TagMetadata(GraphSerializable):
         except StopIteration:
             return None
 
+    def create_next_record(self) -> Union[RDSModel, None]:
+        try:
+            return next(self._records)
+        except StopIteration:
+            return None
+
+    def _create_node_iterator(self) -> Iterator[GraphNode]:
+        node = self.get_node()
+        yield node
+
+    def _create_relation_iterator(self) -> Iterator[GraphRelationship]:
+        return
+        yield
+
+    def _create_record_iterator(self) -> Iterator[RDSModel]:
+        record = self.get_record()
+        yield record
+
 
 # TODO: this should inherit from ProgrammaticDescription in amundsen-common
-class DescriptionMetadata:
+class DescriptionMetadata(GraphSerializable):
     DESCRIPTION_NODE_LABEL = DESCRIPTION_NODE_LABEL_VAL
     PROGRAMMATIC_DESCRIPTION_NODE_LABEL = 'Programmatic_Description'
     DESCRIPTION_KEY_FORMAT = '{description}'
@@ -93,58 +132,76 @@ class DescriptionMetadata:
 
     def __init__(self,
                  text: Optional[str],
-                 source: str = DEFAULT_SOURCE
+                 source: str = DEFAULT_SOURCE,
+                 description_key: Optional[str] = None,
+                 start_label: Optional[str] = None,  # Table, Column, Schema
+                 start_key: Optional[str] = None,
                  ):
         """
         :param source: The unique source of what is populating this description.
         :param text: the description text. Markdown supported.
         """
-        self._source = source
-        self._text = text
+        self.source = source
+        self.text = text
         #  There are so many dependencies on Description node, that it is probably easier to just separate the rest out.
-        if (self._source == self.DEFAULT_SOURCE):
-            self._label = self.DESCRIPTION_NODE_LABEL
+        if self.source == self.DEFAULT_SOURCE:
+            self.label = self.DESCRIPTION_NODE_LABEL
         else:
-            self._label = self.PROGRAMMATIC_DESCRIPTION_NODE_LABEL
+            self.label = self.PROGRAMMATIC_DESCRIPTION_NODE_LABEL
+
+        self.start_label = start_label
+        self.start_key = start_key
+        self.description_key = description_key or self.get_description_default_key(start_key)
+
+        self._node_iter = self._create_node_iterator()
+        self._relation_iter = self._create_relation_iterator()
 
     @staticmethod
     def create_description_metadata(text: Union[None, str],
-                                    source: Optional[str] = DEFAULT_SOURCE
+                                    source: Optional[str] = DEFAULT_SOURCE,
+                                    description_key: Optional[str] = None,
+                                    start_label: Optional[str] = None,  # Table, Column, Schema
+                                    start_key: Optional[str] = None,
                                     ) -> Optional['DescriptionMetadata']:
         # We do not want to create a node if there is no description text!
         if text is None:
             return None
-        if not source:
-            description_node = DescriptionMetadata(text=text, source=DescriptionMetadata.DEFAULT_SOURCE)
-        else:
-            description_node = DescriptionMetadata(text=text, source=source)
+        description_node = DescriptionMetadata(text=text,
+                                               source=source or DescriptionMetadata.DEFAULT_SOURCE,
+                                               description_key=description_key,
+                                               start_label=start_label,
+                                               start_key=start_key)
         return description_node
 
     def get_description_id(self) -> str:
-        if self._source == self.DEFAULT_SOURCE:
+        if self.source == self.DEFAULT_SOURCE:
             return "_description"
         else:
-            return "_" + self._source + "_description"
+            return "_" + self.source + "_description"
 
-    def __repr__(self) -> str:
-        return f'DescriptionMetadata({self._source!r}, {self._text!r})'
+    def get_description_default_key(self, start_key: Optional[str]) -> Optional[str]:
+        return f'{start_key}/{self.get_description_id()}' if start_key else None
 
     def get_node(self, node_key: str) -> GraphNode:
         node = GraphNode(
             key=node_key,
-            label=self._label,
+            label=self.label,
             attributes={
-                DescriptionMetadata.DESCRIPTION_SOURCE: self._source,
-                DescriptionMetadata.DESCRIPTION_TEXT: self._text
+                DescriptionMetadata.DESCRIPTION_SOURCE: self.source,
+                DescriptionMetadata.DESCRIPTION_TEXT: self.text
             }
         )
         return node
 
-    def get_relation(self, start_node: str, start_key: Any, end_key: Any) -> GraphRelationship:
+    def get_relation(self,
+                     start_node: str,
+                     start_key: str,
+                     end_key: str,
+                     ) -> GraphRelationship:
         relationship = GraphRelationship(
             start_label=start_node,
             start_key=start_key,
-            end_label=self._label,
+            end_label=self.label,
             end_key=end_key,
             type=DescriptionMetadata.DESCRIPTION_RELATION_TYPE,
             reverse_type=DescriptionMetadata.INVERSE_DESCRIPTION_RELATION_TYPE,
@@ -152,6 +209,40 @@ class DescriptionMetadata:
         )
         return relationship
 
+    def create_next_node(self) -> Optional[GraphNode]:
+        # return the string representation of the data
+        try:
+            return next(self._node_iter)
+        except StopIteration:
+            return None
+
+    def create_next_relation(self) -> Optional[GraphRelationship]:
+        try:
+            return next(self._relation_iter)
+        except StopIteration:
+            return None
+
+    def _create_node_iterator(self) -> Iterator[GraphNode]:
+        if not self.description_key:
+            raise Exception('Required description node key cannot be None')
+        yield self.get_node(self.description_key)
+
+    def _create_relation_iterator(self) -> Iterator[GraphRelationship]:
+        if not self.start_label:
+            raise Exception('Required relation start node label cannot be None')
+        if not self.start_key:
+            raise Exception('Required relation start key cannot be None')
+        if not self.description_key:
+            raise Exception('Required relation end key cannot be None')
+        yield self.get_relation(
+            start_node=self.start_label,
+            start_key=self.start_key,
+            end_key=self.description_key
+        )
+
+    def __repr__(self) -> str:
+        return f'DescriptionMetadata({self.source!r}, {self.text!r})'
+
 
 class ColumnMetadata:
     COLUMN_NODE_LABEL = 'Column'
@@ -190,7 +281,7 @@ class ColumnMetadata:
                f'{self.sort_order!r}, {self.badges!r})'
 
 
-class TableMetadata(GraphSerializable):
+class TableMetadata(GraphSerializable, TableSerializable):
     """
     Table metadata that contains columns. It implements Neo4jCsvSerializable so that it can be serialized to produce
     Table, Column and relation of those along with relationship with table and schema. Additionally, it will create
@@ -232,6 +323,7 @@ class TableMetadata(GraphSerializable):
     # Only for deduping database, cluster, and schema (table and column will be always processed)
     serialized_nodes_keys: Set[Any] = set()
     serialized_rels_keys: Set[Any] = set()
+    serialized_records_keys: Set[Any] = set()
 
     def __init__(self,
                  database: str,
@@ -273,6 +365,7 @@ class TableMetadata(GraphSerializable):
 
         self._node_iterator = self._create_next_node()
         self._relation_iterator = self._create_next_relation()
+        self._record_iterator = self._create_record_iterator()
 
     def __repr__(self) -> str:
         return f'TableMetadata({self.database!r}, {self.cluster!r}, {self.schema!r}, {self.name!r} ' \
@@ -341,7 +434,8 @@ class TableMetadata(GraphSerializable):
         # Create the table tag nodes
         if self.tags:
             for tag in self.tags:
-                yield TagMetadata.create_tag_node(tag)
+                tag_node = TagMetadata(tag).get_node()
+                yield tag_node
 
         for col in self.columns:
             column_node = GraphNode(
@@ -502,3 +596,106 @@ class TableMetadata(GraphSerializable):
             if (rel_tuple.start_key, rel_tuple.end_key, rel_tuple.type) not in TableMetadata.serialized_rels_keys:
                 TableMetadata.serialized_rels_keys.add((rel_tuple.start_key, rel_tuple.end_key, rel_tuple.type))
                 yield rel_tuple
+
+    def create_next_record(self) -> Union[RDSModel, None]:
+        try:
+            return next(self._record_iterator)
+        except StopIteration:
+            return None
+
+    def _create_record_iterator(self) -> Iterator[RDSModel]:
+        # Database, Cluster, Schema
+        others: List[RDSModel] = [
+            RDSDatabase(
+                rk=self._get_database_key(),
+                name=self.database
+            ),
+            RDSCluster(
+                rk=self._get_cluster_key(),
+                name=self.cluster,
+                database_rk=self._get_database_key()
+            ),
+            RDSSchema(
+                rk=self._get_schema_key(),
+                name=self.schema,
+                cluster_rk=self._get_cluster_key()
+            )
+        ]
+
+        for record in others:
+            if record.rk not in TableMetadata.serialized_records_keys:
+                TableMetadata.serialized_records_keys.add(record.rk)
+                yield record
+
+        # Table
+        yield RDSTable(
+            rk=self._get_table_key(),
+            name=self.name,
+            is_view=self.is_view,
+            schema_rk=self._get_schema_key()
+        )
+
+        # Table description
+        if self.description:
+            description_record_key = self._get_table_description_key(self.description)
+            if self.description.label == DescriptionMetadata.DESCRIPTION_NODE_LABEL:
+                yield RDSTableDescription(
+                    rk=description_record_key,
+                    description_source=self.description.source,
+                    description=self.description.text,
+                    table_rk=self._get_table_key()
+                )
+            else:
+                yield RDSTableProgrammaticDescription(
+                    rk=description_record_key,
+                    description_source=self.description.source,
+                    description=self.description.text,
+                    table_rk=self._get_table_key()
+                )
+
+        # Tag
+        for tag in self.tags:
+            tag_record = TagMetadata(tag).get_record()
+            yield tag_record
+
+            table_tag_record = RDSTableTag(
+                table_rk=self._get_table_key(),
+                tag_rk=TagMetadata.get_tag_key(tag)
+            )
+            yield table_tag_record
+
+        # Column
+        for col in self.columns:
+            yield RDSTableColumn(
+                rk=self._get_col_key(col),
+                name=col.name,
+                type=col.type,
+                sort_order=col.sort_order,
+                table_rk=self._get_table_key()
+            )
+
+            if col.description:
+                description_record_key = self._get_col_description_key(col, col.description)
+                yield RDSColumnDescription(
+                    rk=description_record_key,
+                    description_source=col.description.source,
+                    description=col.description.text,
+                    column_rk=self._get_col_key(col)
+                )
+
+            if col.badges:
+                badge_metadata = BadgeMetadata(
+                    start_label=ColumnMetadata.COLUMN_NODE_LABEL,
+                    start_key=self._get_col_key(col),
+                    badges=col.badges
+                )
+
+                badge_records = badge_metadata.get_badge_records()
+                for badge_record in badge_records:
+                    yield badge_record
+
+                    column_badge_record = RDSColumnBadge(
+                        column_rk=self._get_col_key(col),
+                        badge_rk=badge_record.rk
+                    )
+                    yield column_badge_record
