@@ -3,7 +3,7 @@
 
 import abc
 from typing import (
-    Any, Iterable, Optional,
+    Any, Iterable, Iterator, List, Optional,
 )
 
 from pyhocon import ConfigTree
@@ -41,7 +41,10 @@ class NoopTransformer(Transformer):
 
 class ChainedTransformer(Transformer):
     """
-    A chained transformer that iterates transformers and transforms a record
+    A chained transformer that iterates transformers and transforms a record.
+    Transfomers implemented using generator functons can yield multiple records,
+    which all get passed to the next transformer.
+    Returning None from a transformer filters the record out.
     """
 
     def __init__(self,
@@ -56,13 +59,21 @@ class ChainedTransformer(Transformer):
                 transformer.init(Scoped.get_scoped_conf(conf, transformer.get_scope()))
 
     def transform(self, record: Any) -> Any:
+        records = [record]
         for t in self.transformers:
-            record = t.transform(record)
-            # Check filtered record
-            if not record:
-                return None
+            new_records: List[Any] = []
+            for r in records:
+                result = t.transform(r)
+                # Get all records if the transformer returns an Iterator.
+                if isinstance(result, Iterator):
+                    new_records += list(result)
 
-        return record
+                # Filter the record if it is None
+                elif result is not None:
+                    new_records.append(result)
+            records = new_records
+
+        yield from records
 
     def get_scope(self) -> str:
         return 'transformer.chained'
