@@ -2,9 +2,9 @@
 # SPDX-License-Identifier: Apache-2.0
 
 import logging
+from typing import Any, Dict
 
-from pyhocon import ConfigTree  # noqa: F401
-from typing import Any, Dict  # noqa: F401
+from pyhocon import ConfigTree
 
 from databuilder.transformer.base_transformer import Transformer
 
@@ -19,13 +19,11 @@ class GenericTransformer(Transformer):
     A generic transformer that accepts a callback function that transforms the record on specified field.
     """
 
-    def init(self, conf):
-        # type: (ConfigTree) -> None
+    def init(self, conf: ConfigTree) -> None:
         self._callback_function = conf.get(CALLBACK_FUNCTION)
         self._field_name = conf.get_string(FIELD_NAME)
 
-    def transform(self, record):
-        # type: (Dict[str, Any]) -> Dict[str, Any]
+    def transform(self, record: Dict[str, Any]) -> Dict[str, Any]:
 
         for k, v in record.items():
             if k == self._field_name:
@@ -33,6 +31,5 @@ class GenericTransformer(Transformer):
                 record[k] = new_val
         return record
 
-    def get_scope(self):
-        # type: () -> str
+    def get_scope(self) -> str:
         return 'transformer.generic'
